@@ -6,7 +6,7 @@ from functools import partial
 from itertools import groupby
 
 from trytond.i18n import gettext
-from trytond.model import ModelSQL, ModelView, fields
+from trytond.model import Index, ModelSQL, ModelView, fields
 from trytond.model.exceptions import AccessError
 from trytond.modules.company.model import (
     employee_field, reset_employee, set_employee)
@@ -27,7 +27,7 @@ class PurchaseRequest(ModelSQL, ModelView):
     __name__ = 'purchase.request'
 
     product = fields.Many2One(
-        'product.product', "Product", select=True, readonly=True,
+        'product.product', "Product", readonly=True,
         domain=[('purchasable', '=', True)],
         context={
             'company': Eval('company', -1),
@@ -36,15 +36,15 @@ class PurchaseRequest(ModelSQL, ModelView):
     description = fields.Text('Description', readonly=True, states=STATES)
     summary = fields.Function(fields.Char('Summary'), 'on_change_with_summary')
     party = fields.Many2One(
-        'party.party', "Party", select=True, states=STATES,
+        'party.party', "Party", states=STATES,
         context={
             'company': Eval('company', -1),
             },
         depends={'company'})
     quantity = fields.Float('Quantity', required=True, states=STATES,
         digits=(16, Eval('uom_digits', 2)))
-    uom = fields.Many2One('product.uom', 'UOM', select=True,
-        ondelete='RESTRICT',
+    uom = fields.Many2One(
+        'product.uom', "UOM", ondelete='RESTRICT',
         domain=[
             If(Bool(Eval('product_uom_category')),
                 ('category', '=', Eval('product_uom_category')),
@@ -96,11 +96,17 @@ class PurchaseRequest(ModelSQL, ModelView):
             ('done', "Done"),
             ('cancelled', "Cancelled"),
             ('exception', "Exception"),
-            ], "State", required=True, readonly=True, select=True, sort=False)
+            ], "State", required=True, readonly=True, sort=False)
 
     @classmethod
     def __setup__(cls):
         super(PurchaseRequest, cls).__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.add(
+            Index(
+                t,
+                (t.state, Index.Equality()),
+                where=t.state.in_(['draft', 'purchased', 'exception'])))
         cls._order[0] = ('id', 'DESC')
         cls._buttons.update({
                 'handle_purchase_cancellation_exception': {
@@ -194,22 +200,14 @@ class PurchaseRequest(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        _, operator, value = clause
-        if operator.startswith('!') or operator.startswith('not'):
-            bool_op = 'AND'
-        else:
-            bool_op = 'OR'
-        domain = [bool_op]
-        if value is not None:
-            names = value.split('@', 1)
-            sub_domain = [('product.template.name', operator, names[0])]
-            if len(names) != 1 and names[1]:
-                sub_domain.append(('warehouse', operator, names[1]))
-                if bool_op == 'AND':
-                    sub_domain.insert(0, 'OR')
-            domain.append(sub_domain)
-        domain.append(('description', *clause[1:]))
-        return domain
+        res = []
+        names = clause[2].split('@', 1)
+        res.append(('product.template.name', clause[1], names[0]))
+        if len(names) != 1 and names[1]:
+            res.append(('warehouse', clause[1], names[1]))
+        return ['OR', res,
+            ('description',) + tuple(clause[1:]),
+            ]
 
     @staticmethod
     def default_company():
@@ -479,14 +477,8 @@ class CreatePurchase(Wizard):
 
         requests = [r for r in requests if not r.purchase_line]
 
-        def _sort_keyfunc(requests, request):
-            return (
-                self._group_purchase_key(requests, request)
-                + self._group_purchase_line_key(request))
-        sort_keyfunc = partial(_sort_keyfunc, requests)
-
         keyfunc = partial(self._group_purchase_key, requests)
-        requests = sorted(requests, key=sortable_values(sort_keyfunc))
+        requests = sorted(requests, key=sortable_values(keyfunc))
 
         purchases = []
         lines = []
